
package weka.gui.experiment;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import weka.core.Attribute;
import weka.core.Instance;
import weka.core.converters.CSVLoader;
import weka.experiment.DatabaseResultListener;
import weka.experiment.Experiment;
import java.io.*;
import weka.experiment.CSVResultListener;
import weka.core.Instances;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import java.awt.event.ActionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import structure.DataSet;

/** 
 * @author Alexandre Kouzntsov, uOttawa
 */

  public class VisualizationPanel
  extends JPanel
  implements ActionListener {   
   
 /** An experiment (used for identifying a result source) */
  protected Experiment m_Exp;
  
  /** The list model used for dataset files */
  protected DefaultListModel m_Datasets = new DefaultListModel();
  protected DefaultListModel m_Datasets_Copy = new DefaultListModel();
  
  /** The list model used for Algorithms*/
  protected DefaultListModel m_AlgorithmListModel = new DefaultListModel();
  protected DefaultListModel m_AlgorithmListModel_Copy = new DefaultListModel();

  /** The instances we're extracting results from. */
  protected Instances m_Instances;
  
     // plot area dimensions
  protected int m_plotAreaWidth = 412;
  protected int m_plotAreaHeight = 284;
  
  protected JButton m_DeleteDatasetsBut = new JButton("Exclude");
  protected JButton m_ResetDatasetsBut = new JButton("Reset");       
 
  protected JButton m_DeleteAlgorithmsBut = new JButton("Exclude");
  protected JButton m_ResetAlgorithmsBut = new JButton("Reset");
  
  protected JButton m_DeleteMeasuresBut = new JButton("Exclude");  
  protected JButton m_ResetMeasuresBut = new JButton("Reset");
  
  protected JPanel m_DatasetsPanel = new JPanel(); 
  protected JPanel m_AlgorithmsPanel = new JPanel(); 
  
  protected int m_MeasuresCount;
  protected int m_AlgorithmsCount;
  protected int m_DatasetsCount;
  
  //Mode
  protected JRadioButton m_SimpleModeRBut = 
    new JRadioButton("Not Included"); //Simple
  protected JRadioButton m_AdvancedModeRBut = 
    new JRadioButton("Included");//Advanced
  protected JPanel m_ModePanel = new JPanel();  
  protected JPanel m_MeasurePanel = new JPanel();
  
  //Distance Metrix
  String[] m_MetricsArr = {
         "euclidean",
         "manhattan",
         "maximum"
  };
  protected String m_CurrentMetric="euclidean";
  protected JComboBox m_Metric = new JComboBox(m_MetricsArr);
  protected JPanel m_MetricPanel = new JPanel();
  
  //Projection
  String[] m_ProjectionArr ={
      "Algorithms",
      "Data Sets",
      "Measures"
  }; 
  protected JComboBox m_Projection = new JComboBox(m_ProjectionArr);
  protected JPanel m_ProjectionPanel = new JPanel();  

JList m_list_datasets;
 
String[] m_MeasuresArr ={
      "Accuracy",
      "Error Rate",
      "AUC",
      "1-AUC",
      "RMSE",
      "TPR",
      "FPR",
      "Precision", 
      "Recall",
      "F-Measure",
      "KBrelative"
  };
protected DefaultListModel m_Measures = new DefaultListModel();
protected JList m_Measures_list; 
protected JList m_list_Algorithms;

  // separate panels for rendering axis information
  private AxisPanel m_xAxisPanel;
  private AxisPanel m_yAxisPanel; 
  
  private boolean  m_AccuracyIncluded;
  private boolean  m_ERIncluded;
  private boolean  m_AUC_Included;
  private boolean  m_AUC1_Included;
  private boolean  m_RMSE_Included;
  private boolean  m_TPR_Included;
  private boolean  m_FPR_Included;
  private boolean  m_PrecisionIncluded;
  private boolean  m_RecallIncluded;
  private boolean  m_FMeasureIncluded;
  private boolean  m_KB_Included;
  
  protected double[][]  m_Accuracy;
  protected double[][]  m_ER;
  private double[][]  m_AUC;
  private double[][]  m_AUC1;
  private double[][]  m_RMSE;
  private double[][]  m_TPR;
  private double[][]  m_FPR;
  private double[][]  m_Precision;
  private double[][]  m_Recall;
  private double[][]  m_FMeasure;
  private double[][]  m_KB;
  private double[][]  m_TP;
  private double[][]  m_TN;
  private double[][]  m_FP;
  private double[][]  m_FN;
  
    /** Frame for the Visualization */
  protected JFrame m_NotesFrame = new JFrame("Visualization");
  
  protected DataSet dataset; 
  protected GrPanel pane;  
  protected ButtonGroup m_group;
  protected String m_mode="advanced";
  protected String m_modeRestore="advanced";
  protected String m_focusON="Algorithms";
  protected int  m_linesCounter=0;
  protected String m_duplicates="";

  private class AxisPanel //May be we would not it at all...now Axeses are  just blank
    extends JPanel {
    private boolean m_vertical = false;
    private final int PAD = 5;
    private FontMetrics m_fontMetrics;
    private int m_fontHeight;
    
    public AxisPanel(boolean vertical) {
      m_vertical = vertical;
      this.setBackground(Color.black);
      String fontFamily = this.getFont().getFamily();
      Font newFont = new Font(fontFamily, Font.PLAIN, 10);
      this.setFont(newFont);
    }
    public Dimension getPreferredSize() {
      if (m_fontMetrics == null) {          
	Graphics g = this.getGraphics(); 
	m_fontMetrics = g.getFontMetrics();
	m_fontHeight = m_fontMetrics.getHeight();
      }
      if (!m_vertical) {
	return new Dimension(this.getSize().width, PAD+2+m_fontHeight);
      }
      return new Dimension(50, this.getSize().height);
    }
    public void paintComponent(Graphics g) {
      super.paintComponent(g);
      this.setBackground(Color.white);
      if (m_fontMetrics == null) {
	m_fontMetrics = g.getFontMetrics();
	m_fontHeight = m_fontMetrics.getHeight();
      }
      Dimension d = this.getSize();
      Dimension d2 = m_NotesFrame.getSize();
      g.setColor(Color.gray);
      int hf = m_fontMetrics.getAscent();
      if (!m_vertical) {
	g.drawLine(d.width, PAD, d.width-d2.width, PAD);	
      } else {
	g.drawLine(d.width-PAD, 0, d.width-PAD, d2.height);	
      }
    }
  }
  
  /**
   * Creates the Visualization Panel.
   */  
  public VisualizationPanel(Experiment exp) {  
      
      setExperiment(exp);   
     
      // Initialize the list with items  
      for (int i=0; i<m_MeasuresArr.length; i++) {
        m_Measures.add(i, m_MeasuresArr[i]);
      }
      m_Measures_list = new JList(m_Measures); 
      m_MeasuresCount=m_Measures_list.getModel().getSize();
     
      
      setList();
      m_AlgorithmsCount=m_list_Algorithms.getModel().getSize();
       m_DatasetsCount=m_list_datasets.getModel().getSize();      

     
    JPanel optionsArea = new JPanel();
      
    JPanel classifiersRButtons = new JPanel();    
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();    
    classifiersRButtons.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
    classifiersRButtons.setLayout(gb);
    
    constraints.gridx=0;constraints.gridy=0;constraints.weightx=5;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridwidth=1;constraints.gridheight=1;
    constraints.insets = new Insets(0,2,0,2);
     constraints.anchor=GridBagConstraints.LAST_LINE_START;
     classifiersRButtons.add(m_DeleteAlgorithmsBut,constraints);
     
    constraints.gridx=1;constraints.gridy=0;constraints.weightx=5;
    constraints.gridwidth=1;constraints.gridheight=1; 
     constraints.anchor=GridBagConstraints.LAST_LINE_END;
    classifiersRButtons.add(m_ResetAlgorithmsBut,constraints);    
       
    m_AlgorithmsPanel.setBorder(BorderFactory.createTitledBorder("Algorithms"));
    JScrollPane scrollPaneClassifiers = new JScrollPane(m_list_Algorithms);      
    m_AlgorithmsPanel.setLayout(new BorderLayout());
    m_AlgorithmsPanel.add(scrollPaneClassifiers,BorderLayout.CENTER);
    m_AlgorithmsPanel.add(classifiersRButtons, BorderLayout.SOUTH);
    m_list_Algorithms.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
     
    
    JPanel domainRButtons = new JPanel();    
    gb = new GridBagLayout();
    constraints = new GridBagConstraints();    
    domainRButtons.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
    domainRButtons.setLayout(gb); 
    
    constraints.gridx=0;constraints.gridy=0;constraints.weightx=5;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridwidth=1;constraints.gridheight=1;
    constraints.insets = new Insets(0,2,0,2);
    constraints.anchor=GridBagConstraints.LAST_LINE_START;
    domainRButtons.add(m_DeleteDatasetsBut, constraints);
    constraints.gridx=1;constraints.gridy=0;constraints.weightx=5;
    constraints.gridwidth=1;constraints.gridheight=1; 
    constraints.anchor=GridBagConstraints.LAST_LINE_END;
    domainRButtons.add(m_ResetDatasetsBut, constraints);
        
       
    m_DatasetsPanel.setBorder(BorderFactory.createTitledBorder("Datasets"));
    JScrollPane scrollPaneDatasets = new JScrollPane(m_list_datasets);     
    m_DatasetsPanel.setLayout(new BorderLayout());
    m_DatasetsPanel.add(scrollPaneDatasets,BorderLayout.CENTER);
    m_DatasetsPanel.add(domainRButtons, BorderLayout.SOUTH);
    m_list_datasets.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
       
    
    JPanel measuresButtons = new JPanel();   
    gb = new GridBagLayout();
    constraints = new GridBagConstraints();    
    measuresButtons.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
    measuresButtons.setLayout(gb);    
    constraints.gridx=0;constraints.gridy=0;constraints.weightx=5;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.gridwidth=1;constraints.gridheight=1;
    constraints.insets = new Insets(0,2,0,2);
     constraints.anchor=GridBagConstraints.LAST_LINE_START;
    measuresButtons.add(m_DeleteMeasuresBut, constraints);
    constraints.gridx=1;constraints.gridy=0;constraints.weightx=5;
    constraints.gridwidth=1;constraints.gridheight=1; 
     constraints.anchor=GridBagConstraints.LAST_LINE_END;
    measuresButtons.add(m_ResetMeasuresBut, constraints); 
    JScrollPane scrollPaneMeasurePanel = new JScrollPane(m_Measures_list);    
    m_MeasurePanel.setBorder(BorderFactory.createTitledBorder("Mesures"));
    m_MeasurePanel.setLayout(new BorderLayout());
    m_MeasurePanel.add(scrollPaneMeasurePanel,BorderLayout.CENTER);
    m_MeasurePanel.add(measuresButtons, BorderLayout.SOUTH);
    m_Measures_list.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
    
   optionsArea.setLayout(new BoxLayout(optionsArea,BoxLayout.X_AXIS ));
      optionsArea.add(m_AlgorithmsPanel);
      optionsArea.add(m_DatasetsPanel);
      optionsArea.add(m_MeasurePanel);
      
       m_DeleteMeasuresBut.setEnabled(false);
      m_DeleteDatasetsBut.setEnabled(false);
      m_DeleteAlgorithmsBut.setEnabled(false);   
      m_DeleteMeasuresBut.addActionListener(this);
      m_DeleteDatasetsBut.addActionListener(this);
      m_DeleteAlgorithmsBut.addActionListener(this);
      
      m_ResetMeasuresBut.setEnabled(false);
      m_ResetDatasetsBut.setEnabled(false);
      m_ResetAlgorithmsBut.setEnabled(false);   
      m_ResetMeasuresBut.addActionListener(this);
      m_ResetDatasetsBut.addActionListener(this);
      m_ResetAlgorithmsBut.addActionListener(this);
      
      m_AdvancedModeRBut.setSelected(true);
      m_SimpleModeRBut.addActionListener(m_RadioListener);
      m_AdvancedModeRBut.addActionListener(m_RadioListener);
      
      m_group = new ButtonGroup();
      m_group.add(m_SimpleModeRBut);
      m_group.add(m_AdvancedModeRBut);
      m_SimpleModeRBut.setActionCommand("simple");
      m_AdvancedModeRBut.setActionCommand("advanced");
      
      m_Metric.addActionListener(m_JBOX_Metric_Listener);
      m_Projection.addActionListener(m_JBOX_Projection_Listener);    
    
    JPanel gfxPanel = new JPanel();
    gfxPanel.setLayout(new BorderLayout());    
    gfxPanel.setBorder(BorderFactory.createTitledBorder("Graphics"));
   
    pane=new GrPanel((int)m_plotAreaWidth,(int)m_plotAreaHeight);
    gfxPanel.add(pane, BorderLayout.CENTER);
                pane.runMDS(m_mode,m_CurrentMetric, m_linesCounter, m_list_Algorithms.getModel().getSize(),m_list_datasets.getModel().getSize() ,m_Measures_list.getModel().getSize(), m_duplicates);
		pane.repaint();
                
                
    m_xAxisPanel = new AxisPanel(false);
    gfxPanel.add(m_xAxisPanel, BorderLayout.SOUTH);
    m_yAxisPanel = new AxisPanel(true);
    gfxPanel.add(m_yAxisPanel, BorderLayout.WEST);
    
    JPanel containerPanel = new JPanel();
    containerPanel.setLayout(new BorderLayout());
    containerPanel.add(gfxPanel, BorderLayout.CENTER);
    
    
    m_ModePanel.setBorder(BorderFactory.createTitledBorder("Ideal Node"));
    m_ModePanel.setLayout(new GridLayout(2,1));
    m_ModePanel.add(m_SimpleModeRBut);
    m_ModePanel.add(m_AdvancedModeRBut);
   
    m_MetricPanel.setBorder(BorderFactory.createTitledBorder("Distance Metrics"));
    m_MetricPanel.add(m_Metric, BorderLayout.CENTER);
    m_ProjectionPanel.setBorder(BorderFactory.createTitledBorder("Focused On"));
    m_ProjectionPanel.add(m_Projection, BorderLayout.CENTER);
    
    JPanel rightHandToolsPanel = new JPanel(); //this panel contains the widgets to the right of the BoundaryPanel.
    rightHandToolsPanel.setLayout(new BoxLayout(rightHandToolsPanel, BoxLayout.PAGE_AXIS));
    rightHandToolsPanel.add(m_ModePanel);
    rightHandToolsPanel.add(m_MetricPanel);
    rightHandToolsPanel.add(m_ProjectionPanel);    
    
    containerPanel.add(rightHandToolsPanel, BorderLayout.EAST);    
    
    m_NotesFrame.setSize(800, 600); 
    m_NotesFrame.setVisible(true);
     m_NotesFrame.add(containerPanel, BorderLayout.CENTER);    
    m_NotesFrame.add(optionsArea, BorderLayout.SOUTH);
    
  }
  /**
   * Tells the panel to use a new experiment.
   *
   * @param exp a value of type 'Experiment'
   */
  public void setExperiment(Experiment exp) {    
    m_Exp = exp;    
  }
  
   
  
  public void resetAlgoithms() {     
      m_AlgorithmListModel.removeAllElements();
      m_AlgorithmsCount=m_AlgorithmListModel_Copy.getSize();
      for(int i=0; i<m_AlgorithmsCount;i++){
         m_AlgorithmListModel.addElement(m_AlgorithmListModel_Copy.get(i));
         
         m_list_Algorithms.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
      }        
  }
  
public void resetDatasets() {     
    m_Datasets.removeAllElements();
    m_DatasetsCount=m_Datasets_Copy.getSize();
      for(int i=0; i<m_DatasetsCount;i++){
         m_Datasets.addElement(m_Datasets_Copy.get(i));
      }  
    m_list_datasets.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
   }
  
  
  
  
   /**
   * Loads results from a set of instances contained in the supplied
   * file.
   *
   * @param f a value of type 'File'
   */
  protected void setInstancesFromFile(File f) {

    String fileType = f.getName();
    try {
     // m_FromLab.setText("Reading from file...");
      if (f.getName().toLowerCase().endsWith(Instances.FILE_EXTENSION)) {	    
	fileType = "arff";
	Reader r = new BufferedReader(new FileReader(f));
	setInstances(new Instances(r));
	r.close();
      } else if (f.getName().toLowerCase().endsWith(CSVLoader.FILE_EXTENSION)) {
	fileType = "csv";
	CSVLoader cnv = new CSVLoader();
	cnv.setSource(f);
	Instances inst = cnv.getDataSet();
	setInstances(inst);
      } else {
	throw new Exception("Unrecognized file type");
      }
    } catch (Exception ex) {
   // to add exeption handler
      }
    }  
  
    /**
   * Sets up the panel with a new set of instances, attempting
   * to guess the correct settings for various columns.
   *
   * @param newInstances the new set of results.
   */
  public void setInstances(Instances newInstances) { 
   m_Instances = newInstances;    
   int numAlg=0;
   int  numDat=0;
   
   for (int i = 0; i < m_Instances.numAttributes(); i++) {
      String name = m_Instances.attribute(i).name();
      
      if (name.equals("Key_Scheme")) {
          numAlg=m_Instances.attribute(i).numValues();
      }   
      if (name.equals("Key_Dataset")) {
          numDat=m_Instances.attribute(i).numValues(); 
      }    
   }
    String[] tempAr=new String[numAlg];
    String[] tempDataAr=new String[numDat];
    
    for (int i = 0; i < m_Instances.numAttributes(); i++) {
      String name = m_Instances.attribute(i).name();     

      if (name.equals("Key_Dataset")) {	
        for(int j=0; j<m_Instances.attribute(i).numValues();j++){
           tempDataAr[j]=m_Instances.attribute(i).value(j);       
        } 
      } else if (name.equals("Key_Scheme")) {	
        for(int j=0; j<m_Instances.attribute(i).numValues();j++){
            tempAr[j]=m_Instances.attribute(i).value(j);
        }
      } else if (name.equals("Key_Scheme_options")) {
      } 
    }     
    // Initialize the list with items    
    for (int i=0; i<tempAr.length; i++) {
        m_AlgorithmListModel.add(i, tempAr[i]);
        m_AlgorithmListModel_Copy.add(i, tempAr[i]);
    }
     for (int i=0; i<tempDataAr.length; i++) {
        m_Datasets.add(i, tempDataAr[i]);
        m_Datasets_Copy.add(i, tempDataAr[i]);
    }
    m_list_Algorithms= new JList(m_AlgorithmListModel);
    m_list_datasets= new JList(m_Datasets);
    resetGraph();          
  }  

  /**
   * Handle actions when buttons get pressed.
   *
   * @param e a value of type 'ActionEvent'
   */
  public void actionPerformed(ActionEvent e) {    

    if (e.getSource() == m_DeleteMeasuresBut) {
           // Delete the selected Measures
      int [] selected = m_Measures_list.getSelectedIndices(); 
      if (selected != null) {
	for (int i = selected.length - 1; i >= 0; i--) {
	  int current = selected[i];
          m_Measures.remove(current);
	  if (m_Measures_list.getModel().getSize() > current) {
	    m_Measures_list.setSelectedIndex(current);
	  } else {
	    m_Measures_list.setSelectedIndex(current - 1);
	  }
	}
      }
      setButtons(null);
       resetGraph();
    }
    else if (e.getSource() == m_DeleteAlgorithmsBut) {
           // Delete the selected Algorithms        
      int [] selected = m_list_Algorithms.getSelectedIndices(); 
      if (selected != null) {
	for (int i = selected.length - 1; i >= 0; i--) {
	  int current = selected[i];         
          m_AlgorithmListModel.remove(current);         
	  if (m_list_Algorithms.getModel().getSize() > current) {
	    m_list_Algorithms.setSelectedIndex(current);
	  } else {
	    m_list_Algorithms.setSelectedIndex(current - 1);
	  }
	}
      }   
      setButtons(null);
       resetGraph();
    }
    else if (e.getSource() == m_DeleteDatasetsBut) {
           // Delete the selected Datasets
      int [] selected = m_list_datasets.getSelectedIndices(); 
      if (selected != null) {
	for (int i = selected.length - 1; i >= 0; i--) {
	  int current = selected[i];
          m_Datasets.remove(current);
	  if (m_list_datasets.getModel().getSize() > current) {
	    m_list_datasets.setSelectedIndex(current);
	  } else {
	    m_list_datasets.setSelectedIndex(current - 1);
	  }
	}
      }   
      setButtons(null);
       resetGraph();
    }
    else if (e.getSource() == m_ResetMeasuresBut) {
      m_Measures.removeAllElements();
      for (int i=0; i<m_MeasuresArr.length; i++) {
         m_Measures.add(i, m_MeasuresArr[i]);
      }
     // m_Measures_list = new JList(m_Measures); 
      m_Measures_list.addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          setButtons(e);
        }
      });
      setButtons(null);
      resetGraph();
    
    }
    else if (e.getSource() == m_ResetDatasetsBut) {       
      resetDatasets();     
      setButtons(null);    
      resetGraph();
    }
    else if (e.getSource() == m_ResetAlgorithmsBut) {     
      resetAlgoithms();      
      setButtons(null);
      resetGraph();
    }
    
  }
  
  /**
   * sets the state of the buttons according to the selection state of the
   * JList.
   * 
   * @param e the event   */  
  private void setButtons(ListSelectionEvent e) {
      m_DeleteMeasuresBut.setEnabled(m_Measures_list.getModel().getSize() > 1); 
      m_ResetMeasuresBut.setEnabled(m_MeasuresArr.length>m_Measures_list.getModel().getSize());      
  
     m_DeleteAlgorithmsBut.setEnabled(m_list_Algorithms.getModel().getSize() > 1);     
     m_ResetAlgorithmsBut.setEnabled( m_AlgorithmListModel_Copy.getSize()>m_list_Algorithms.getModel().getSize());
    
     m_DeleteDatasetsBut.setEnabled(m_list_datasets.getModel().getSize() > 1);      
     m_ResetDatasetsBut.setEnabled(m_Datasets_Copy.getSize()>m_list_datasets.getModel().getSize());
   
  }
  
 private void setList() {
    if ((m_Exp.getResultListener() instanceof CSVResultListener)) { 
      File resultFile = ((CSVResultListener) m_Exp.getResultListener())
	.getOutputFile();
      if ((resultFile == null)) {
	//m_FromLab.setText("No result file");
      } else {
	setInstancesFromFile(resultFile);
      }  
    } else if (m_Exp.getResultListener() instanceof DatabaseResultListener) {
      String dbaseURL = ((DatabaseResultListener) m_Exp.getResultListener())
	.getDatabaseURL();
      try {
	//Here should be code for DB
      } catch (Exception ex) {	
      }
    } else {
     // m_FromLab.setText("Can't get results from experiment");
    }
}
 
  /** Handle radio buttons */
  ActionListener m_RadioListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	m_mode=e.getActionCommand() ;
        m_modeRestore=m_mode;
        resetGraph();
      }
    };
     
    ActionListener m_JBOX_Metric_Listener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	JComboBox cb = (JComboBox)e.getSource();
        m_CurrentMetric = (String)cb.getSelectedItem();
        resetGraph();
      }
    };
    
    ActionListener m_JBOX_Projection_Listener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	JComboBox cb = (JComboBox)e.getSource();
         m_focusON= (String)cb.getSelectedItem();
         if(m_focusON.equals("Measures")){
             m_AdvancedModeRBut.setEnabled(false);
             m_mode="simple";
         }
         else{
             m_AdvancedModeRBut.setEnabled(true);
             m_mode=m_modeRestore;
         }
         
         resetGraph();

      }
    };
    
    private void setMeasuresInUse(){    
       m_AccuracyIncluded=false;
       m_ERIncluded=false;
       m_AUC_Included=false;
       m_AUC1_Included=false;
       m_RMSE_Included=false;
       m_TPR_Included=false;
       m_FPR_Included=false;
       m_PrecisionIncluded=false;
       m_RecallIncluded=false;
       m_FMeasureIncluded=false;
        m_KB_Included=false;
        
        m_MeasuresCount=m_Measures_list.getModel().getSize();
        for(int i=0; i<m_MeasuresCount;i++){
            String measure=m_Measures_list.getModel().getElementAt(i).toString();
            if(measure.indexOf("Accuracy")>-1)
                m_AccuracyIncluded=true;
            if(measure.indexOf("Error Rate")>-1)
                m_ERIncluded=true;
            if(measure.indexOf("AUC")>-1)
                m_AUC_Included=true;
            if(measure.indexOf("1-AUC")>-1)
                m_AUC1_Included=true;
            if(measure.indexOf("RMSE")>-1)
                m_RMSE_Included=true;
            if(measure.indexOf("TPR")>-1)
                m_TPR_Included=true;
            if(measure.indexOf("FPR")>-1)
                m_FPR_Included=true;
            if(measure.indexOf("Precision")>-1)
                m_PrecisionIncluded=true;
            if(measure.indexOf("Recall")>-1)
                m_RecallIncluded=true;
            if(measure.indexOf("F-Measure")>-1)
                m_FMeasureIncluded=true;
            if(measure.indexOf("KBrelative")>-1)
                m_KB_Included=true;
        }      
    }
    
    private void setMeasuresArrays(){
         try{
        setMeasuresInUse();
        m_AlgorithmsCount=m_list_Algorithms.getModel().getSize();
        m_DatasetsCount=m_list_datasets.getModel().getSize();
           
        if(m_AccuracyIncluded)
            m_Accuracy=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_ERIncluded)
            m_ER=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_AUC_Included)
            m_AUC=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_AUC1_Included)
            m_AUC1=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_RMSE_Included)
            m_RMSE=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_TPR_Included)
            m_TPR=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_FPR_Included)
            m_FPR=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_PrecisionIncluded)
            m_Precision=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_RecallIncluded)
            m_Recall=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_FMeasureIncluded)
            m_FMeasure=new double[m_AlgorithmsCount][m_DatasetsCount];
        if(m_KB_Included)
            m_KB=new double[m_AlgorithmsCount][m_DatasetsCount];
            m_TP=new double[m_AlgorithmsCount][m_DatasetsCount];
            m_FP=new double[m_AlgorithmsCount][m_DatasetsCount];
            m_TN=new double[m_AlgorithmsCount][m_DatasetsCount];
            m_FN=new double[m_AlgorithmsCount][m_DatasetsCount];    
         }catch(Exception e) {e.printStackTrace();}
    }
    private void fillMeusuresArrays(){ 
        int instancesCount=m_Instances.numInstances();
        String previousData="";
        String currentData="";
        String previousSchema="";
        String currentSchema="";
        int indexAlgorthm=0;
        int indexData=0;
        double[][] normAccuracy=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normER=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normAUC=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normAUC1=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normRMSE=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normTPR=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normFPR=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normPrecision=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normRecall=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normFMeasure=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
        double[][] normKB=new double[m_list_Algorithms.getModel().getSize()][m_list_datasets.getModel().getSize()];
      try{                      
        for(int i=0; i<instancesCount;i++){            
            Instance curInstence=m_Instances.instance(i);
            Attribute atr=m_Instances.attribute("Key_Dataset");
            String dataset=curInstence.stringValue(atr);
            atr=m_Instances.attribute("Key_Scheme");
            String algorithm=curInstence.stringValue(atr);
            atr=m_Instances.attribute("Key_Scheme_options");
            //String options=curInstence.stringValue(atr);              
            if(instanceIncluded(dataset,algorithm)){
                 currentData=dataset;
                 //currentSchema=algorithm+options;
                 currentSchema=algorithm;                 
                 if(!(currentData.equals(previousData))){
                     for(int j=0; j<m_list_datasets.getModel().getSize();j++){
                       String temp=m_list_datasets.getModel().getElementAt(j).toString();
                       //if(temp.indexOf(currentData)>-1)
                       if(temp.equals(currentData))
                           indexData=j;                       
                     }
                 }               
                 if(!(currentSchema.equals(previousSchema))){                    
                     for(int j=0; j<m_list_Algorithms.getModel().getSize();j++){                          
                       String temp=m_list_Algorithms.getModel().getElementAt(j).toString();                       
                       
                        if(temp.equals(algorithm))
                           indexAlgorthm=j;
                     }
                 }  
             if(m_AccuracyIncluded){
                  atr=m_Instances.attribute("Percent_correct");
                  double t=curInstence.value(atr);                 
                  m_Accuracy[indexAlgorthm][indexData]= m_Accuracy[indexAlgorthm][indexData]+t;                    
                  normAccuracy[indexAlgorthm][indexData]++;
             }
                 if(m_ERIncluded){
                  atr=m_Instances.attribute("Percent_correct");
                  double t=curInstence.value(atr);               
                  m_ER[indexAlgorthm][indexData]= m_ER[indexAlgorthm][indexData]+t;                                 
                  normER[indexAlgorthm][indexData]++;
             }                         
            if(m_AUC_Included){
                  atr=m_Instances.attribute("Area_under_ROC");
                  double t=curInstence.value(atr);
                  m_AUC[indexAlgorthm][indexData]= m_AUC[indexAlgorthm][indexData]+t;                  
                  normAUC[indexAlgorthm][indexData]++;
             }   
            if(m_AUC1_Included){
                  atr=m_Instances.attribute("Area_under_ROC");
                  double t=curInstence.value(atr);
                  m_AUC1[indexAlgorthm][indexData]= m_AUC1[indexAlgorthm][indexData]+t;                 
                  normAUC1[indexAlgorthm][indexData]++;
             }   
                         
             if(m_RMSE_Included){
                  atr=m_Instances.attribute("Root_mean_squared_error");
                  double t=curInstence.value(atr);
                  m_RMSE[indexAlgorthm][indexData]= m_RMSE[indexAlgorthm][indexData]+t;                 
                  normRMSE[indexAlgorthm][indexData]++;
             }
                 
             if(m_TPR_Included){
                  atr=m_Instances.attribute("True_positive_rate");
                  double t=curInstence.value(atr);                  
                  m_TPR[indexAlgorthm][indexData]= m_TPR[indexAlgorthm][indexData]+t;                 
                  normTPR[indexAlgorthm][indexData]++;
             }
                 
             if(m_FPR_Included){
                  atr=m_Instances.attribute("False_positive_rate");
                  double t=curInstence.value(atr);
                  m_FPR[indexAlgorthm][indexData]= m_FPR[indexAlgorthm][indexData]+t;                 
                  normFPR[indexAlgorthm][indexData]++;
             }
                 
             if(m_PrecisionIncluded){
                  atr=m_Instances.attribute("IR_precision");
                  double t=curInstence.value(atr);
                  m_Precision[indexAlgorthm][indexData]= m_Precision[indexAlgorthm][indexData]+t;                 
                  normPrecision[indexAlgorthm][indexData]++;
             }
                 
             if(m_RecallIncluded){
                  atr=m_Instances.attribute("IR_recall");
                  double t=curInstence.value(atr);
                  m_Recall[indexAlgorthm][indexData]= m_Recall[indexAlgorthm][indexData]+t;                 
                  normRecall[indexAlgorthm][indexData]++;
             }
                 
             if(m_FMeasureIncluded){
                  atr=m_Instances.attribute("F_measure");
                  double t=curInstence.value(atr);
                  m_FMeasure[indexAlgorthm][indexData]= m_FMeasure[indexAlgorthm][indexData]+t;                   
                  normFMeasure[indexAlgorthm][indexData]++;
             }
             
             if(m_KB_Included){
                  atr=m_Instances.attribute("KB_relative_information");
                  double t=curInstence.value(atr);
                  m_KB[indexAlgorthm][indexData]= m_KB[indexAlgorthm][indexData]+t;
                  
                  atr=m_Instances.attribute("Num_true_positives");
                  t=curInstence.value(atr);
                  m_TP[indexAlgorthm][indexData]= t;
                  
                  atr=m_Instances.attribute("Num_false_positives");
                  t=curInstence.value(atr);
                  m_FP[indexAlgorthm][indexData]= t;
                  
                  atr=m_Instances.attribute("Num_true_negatives");
                  t=curInstence.value(atr);
                  m_TN[indexAlgorthm][indexData]= t;
                  
                  atr=m_Instances.attribute("Num_false_negatives");
                  t=curInstence.value(atr);
                  m_FN[indexAlgorthm][indexData]= t;
                  normKB[indexAlgorthm][indexData]++;
                  
             }                 
                 previousData=dataset;
                 //previousSchema=algorithm+options;
                 previousSchema=algorithm;                 
            }         
        }           
            for(int i=0; i<m_AlgorithmsCount;i++){
             for(int j=0; j<m_DatasetsCount;j++){
                 if(m_AccuracyIncluded)
                     m_Accuracy[i][j]=m_Accuracy[i][j]/(normAccuracy[i][j]*100);
                  if(m_ERIncluded)
                     m_ER[i][j]=1-m_ER[i][j]/(normER[i][j]*100);
                 if(m_AUC_Included)
                     m_AUC[i][j]=m_AUC[i][j]/normAUC[i][j];
                 if(m_AUC1_Included)
                     m_AUC1[i][j]=1-m_AUC1[i][j]/normAUC1[i][j];
                 if(m_RMSE_Included)
                     m_RMSE[i][j]=m_RMSE[i][j]/normRMSE[i][j];
                 if(m_TPR_Included)
                     m_TPR[i][j]=m_TPR[i][j]/normTPR[i][j];
                 if(m_FPR_Included)
                     m_FPR[i][j]=m_FPR[i][j]/normFPR[i][j];
                 if(m_PrecisionIncluded)
                     m_Precision[i][j]=m_Precision[i][j]/normPrecision[i][j];
                 if(m_RecallIncluded)
                     m_Recall[i][j]=m_Recall[i][j]/normRecall[i][j];
                 if(m_FMeasureIncluded)
                     m_FMeasure[i][j]=m_FMeasure[i][j]/normFMeasure[i][j];
                 if(m_KB_Included){
                     m_KB[i][j]=m_KB[i][j]/(normKB[i][j]*100);
                     m_KB[i][j]=m_KB[i][j]/(m_TP[0][j]+m_FP[0][j]+m_TN[0][j]+m_FN[0][j]);                 
                 }                
             }
         }            
       }catch(Exception e) {e.printStackTrace();}  
    }
    
    private boolean instanceIncluded(String dataset, String algorithm){        
         boolean dataIncluded=false;
         boolean algorithmIncluded=false;
         boolean optionsIncluded=true;   
         m_AlgorithmsCount=m_list_Algorithms.getModel().getSize();  
       
        for(int i=0; i<m_AlgorithmsCount;i++){
            String curAlgorithm=m_list_Algorithms.getModel().getElementAt(i).toString();
            if(curAlgorithm.indexOf(algorithm)>-1)
                algorithmIncluded=true;
        }      
        
        m_DatasetsCount=m_list_datasets.getModel().getSize();  
        
        for(int i=0; i<m_DatasetsCount;i++){
            String curDataset=m_list_datasets.getModel().getElementAt(i).toString();
           
            if(curDataset.indexOf(dataset)>-1)
                dataIncluded=true;
        }
                 
        return(dataIncluded&&algorithmIncluded&&optionsIncluded);
    }
    
    private void resetGraph(){
      setMeasuresArrays();
      fillMeusuresArrays();
      outputFile(); 
      madeHighDemDataFile();
      pane.runMDS(m_mode,m_CurrentMetric,m_linesCounter, m_list_Algorithms.getModel().getSize(),m_list_datasets.getModel().getSize() ,m_Measures_list.getModel().getSize(), m_duplicates);
      pane.repaint();
    }
        
     protected  void madeHighDemDataFile(){
      try{  
          String wekaFileNameR="exportForR.csv";
          FileWriter fstreamR = new FileWriter(wekaFileNameR);
          BufferedWriter outR = new BufferedWriter(fstreamR);
          m_MeasuresCount=m_Measures_list.getModel().getSize();
          m_linesCounter=0;
          String line="";
          int columnsNumber=0;
          int linesNumber=0;
          if(m_focusON.equals("Algorithms")){
              columnsNumber=m_DatasetsCount;
              linesNumber=m_AlgorithmsCount;          
          }
          else if(m_focusON.equals("Data Sets")){
              columnsNumber=m_AlgorithmsCount;
              linesNumber=m_DatasetsCount;          
          }
          else if(m_focusON.equals("Measures")){
              columnsNumber=m_AlgorithmsCount+m_DatasetsCount;
              linesNumber=m_MeasuresCount; 
          }
          
          //Build ideal vector (possible first line of the matrix)
           for(int j=0; j<columnsNumber;j++){  
               
                   if(m_AccuracyIncluded)
                       line=addToLine(line,"1");               
                   if(m_ERIncluded)
                       line=addToLine(line,"0");
                   if(m_AUC_Included)
                       line=addToLine(line,"1");
                   if(m_AUC1_Included)
                       line=addToLine(line,"0");
                   if(m_RMSE_Included)
                       line=addToLine(line,"0");                   
                   if(m_TPR_Included)
                       line=addToLine(line,"0");
                   if(m_FPR_Included)
                       line=addToLine(line,"0");
                   if(m_PrecisionIncluded)
                       line=addToLine(line,"1");
                   if(m_RecallIncluded)
                       line=addToLine(line,"1");
                   if(m_FMeasureIncluded)
                       line=addToLine(line,"1");
                   if(m_KB_Included)
                       line=addToLine(line,"1");
               
             } //end for columnsNumber
          
          if((m_mode.equals("advanced"))&&(!m_focusON.equals("Measures"))){//writing ot the ideal's line
                   outR.write(line);
                   outR.newLine();
                   m_linesCounter++;
               }
          
          //build the rest of matrix
          for(int i=0; i<linesNumber;i++){
            if(i>0)
                outR.newLine();
            line="";
            if(m_focusON.equals("Measures")){
                for(int j=0; j<m_AlgorithmsCount;j++){
                    for(int k=0; k<m_DatasetsCount;k++){
                        line=addToLine(line,getMeasureScore(i, j, k));
                    }
                }            
            }
            else{
              int a=0,b=0;
              for(int j=0; j<columnsNumber;j++){
                  if(m_focusON.equals("Algorithms")){
                      a=i;
                      b=j;
                  }
                  else if(m_focusON.equals("Data Sets")){
                      a=j;
                      b=i;                       
                  }
                  if(m_AccuracyIncluded)
                      line=addToLine(line,m_Accuracy[a][b]);               
                  if(m_ERIncluded)
                      line=addToLine(line,m_ER[a][b]);
                  if(m_AUC_Included)
                      line=addToLine(line,m_AUC[a][b]);
                  if(m_AUC1_Included)
                      line=addToLine(line,m_AUC1[a][b]);
                  if(m_RMSE_Included)
                      line=addToLine(line,m_RMSE[a][b]);                   
                  if(m_TPR_Included)
                      line=addToLine(line,m_TPR[a][b]);
                  if(m_FPR_Included)
                      line=addToLine(line,m_FPR[a][b]);
                  if(m_PrecisionIncluded)
                      line=addToLine(line,m_Precision[a][b]);
                  if(m_RecallIncluded)
                      line=addToLine(line,m_Recall[a][b]);
                  if(m_FMeasureIncluded)
                      line=addToLine(line,m_FMeasure[a][b]);
                  if(m_KB_Included)
                      line=addToLine(line,m_KB[a][b]);
              }
            }//for columnsNumber,  Algorithms or Data Sets
            outR.write(line);
            m_linesCounter++;
          }//for linesNumber
      outR.close(); 
      
      }catch(Exception e) {e.printStackTrace();}     
     }
     
     protected double getMeasureScore(int measureIndex, int classifierIndex, int dataSetIndex){
         String measure=m_Measures_list.getModel().getElementAt(measureIndex).toString();
         double r=0;
         if(measure.equals("Accuracy"))
                r=m_Accuracy[classifierIndex][dataSetIndex];
            if(measure.equals("Error Rate"))
                r=m_ER[classifierIndex][dataSetIndex];
            if(measure.equals("AUC"))
                r=m_AUC[classifierIndex][dataSetIndex];
            if(measure.equals("1-AUC"))
                r=m_AUC1[classifierIndex][dataSetIndex];
            if(measure.equals("RMSE"))
                r=m_RMSE[classifierIndex][dataSetIndex];
            if(measure.equals("TPR"))
                r=m_TPR[classifierIndex][dataSetIndex];
            if(measure.equals("FPR"))
                r=m_FPR[classifierIndex][dataSetIndex];
            if(measure.equals("Precision"))
                r=m_Precision[classifierIndex][dataSetIndex];
            if(measure.equals("Recall"))
                r=m_Recall[classifierIndex][dataSetIndex];
            if(measure.equals("F-Measure"))
                r=m_FMeasure[classifierIndex][dataSetIndex];
            if(measure.equals("KBrelative"))
                r=m_KB[classifierIndex][dataSetIndex]; 
         return(r);
     }
     
     protected String  addToLine(String lineR,String value){
         if(lineR.length()>0)
                       lineR=lineR+",";
                   lineR=lineR+value;
         return(lineR);  
     }
     
     protected String  addToLine(String lineR,Double value){
         if(lineR.length()>0)
                       lineR=lineR+",";
                   lineR=lineR+value.toString();
         return(lineR);  
     }
  
    private void outputFile(){//It is just for debuging...
        try{            
            String wekaFileName="export.csv";
            FileWriter fstream = new FileWriter(wekaFileName);
            BufferedWriter out = new BufferedWriter(fstream); 
            char doublequote='\"';
            String header=""+doublequote+doublequote+","+doublequote+"Algorithm"+doublequote+","+doublequote+"Dataset"+doublequote;
            for(int i=0; i<m_MeasuresCount;i++){
                header=header+","+doublequote+ m_Measures_list.getModel().getElementAt(i)+doublequote;
            }
            out.write(header);
            out.newLine();                       
            int count=1;
            setMeasuresInUse();
           
            for(int i=0; i<m_AlgorithmsCount;i++){
             for(int j=0; j<m_DatasetsCount;j++){
               String line=""+doublequote+count+doublequote+","+doublequote+m_list_Algorithms.getModel().getElementAt(i)+doublequote+","+doublequote+m_list_datasets.getModel().getElementAt(j)+doublequote;
               if(m_AccuracyIncluded)
                   line=line+","+m_Accuracy[i][j];
               if(m_ERIncluded)
                   line=line+","+m_ER[i][j];
               if(m_AUC_Included)
                   line=line+","+m_AUC[i][j];
                if(m_AUC1_Included)
                   line=line+","+m_AUC1[i][j];
               if(m_RMSE_Included)
                   line=line+","+m_RMSE[i][j];
               if(m_TPR_Included)
                   line=line+","+m_TPR[i][j];
               if(m_FPR_Included)
                   line=line+","+m_FPR[i][j];
               if(m_PrecisionIncluded)
                   line=line+","+m_Precision[i][j];
               if(m_RecallIncluded)
                   line=line+","+m_Recall[i][j];
               if(m_FMeasureIncluded)
                   line=line+","+m_FMeasure[i][j];
               if(m_KB_Included)
                   line=line+","+m_KB[i][j];
               out.write(line);
               out.newLine(); 
               if(m_AUC1[i][j]<0){//Actually, m_duplicates is not more in use, could be discard
                   m_duplicates=m_list_Algorithms.getModel().getElementAt(i)+" and "+m_list_datasets.getModel().getElementAt(j);               
               }
               count++;
             }
            }
          out.close(); 
      }catch(Exception e) {e.printStackTrace();}    
    } 
  }  